@@ -21,11 +21,13 @@ from __future__ import division, print_function, unicode_literals
 import time
 import unicodedata
 
+import logging
+_logger = logging.getLogger(__name__)
+
 try:
     from gi.repository import Atspi
 except ImportError as e:
-    _logger.info(_("Atspi unavailable, "
-                   "word suggestions not fully functional"))
+    pass
 
 from Onboard.AtspiStateTracker import AtspiStateTracker, AtspiStateType
 from Onboard.TextDomain        import TextDomains
@@ -36,12 +38,6 @@ from Onboard                   import KeyCommon
 ### Config Singleton ###
 from Onboard.Config import Config
 config = Config()
-########################
-
-### Logging ###
-import logging
-_logger = logging.getLogger(__name__)
-###############
 
 
 class TextContext:
@@ -61,13 +57,13 @@ class TextContext:
     def insert_text(self, offset, text):
         return NotImplementedError()
 
-    def insert_text_at_cursor(self, text):
+    def insert_text_at_caret(self, text):
         return NotImplementedError()
 
     def delete_text(self, offset, length = 1):
         return NotImplementedError()
 
-    def delete_text_before_cursor(self, length = 1):
+    def delete_text_before_caret(self, length = 1):
         return NotImplementedError()
 
     def get_context(self):
@@ -76,7 +72,7 @@ class TextContext:
     def get_line(self):
         raise NotImplementedError()
 
-    def get_line_cursor_pos(self):
+    def get_line_caret_pos(self):
         raise NotImplementedError()
 
     def get_changes(self):
@@ -107,8 +103,8 @@ class AtspiTextContext(TextContext):
 
         self._context = ""
         self._line = ""
-        self._line_cursor = 0
-        self._span_at_cursor = TextSpan()
+        self._line_caret = 0
+        self._span_at_caret = TextSpan()
         self._begin_of_text = False        # context starts at begin of text?
         self._begin_of_text_offset = None  # offset of text begin
 
@@ -129,7 +125,7 @@ class AtspiTextContext(TextContext):
     def get_context(self):
         """
         Returns the predictions context, i.e. some range of
-        text before the cursor position.
+        text before the caret position.
         """
         if self._accessible is None:
             return ""
@@ -163,20 +159,20 @@ class AtspiTextContext(TextContext):
         return self._line \
                if self._accessible else ""
 
-    def get_line_cursor_pos(self):
-        return self._line_cursor \
+    def get_line_caret_pos(self):
+        return self._line_caret \
                if self._accessible else 0
 
-    def get_line_past_cursor(self):
-        return self._line[self._line_cursor:] \
+    def get_line_past_caret(self):
+        return self._line[self._line_caret:] \
                if self._accessible else ""
 
-    def get_span_at_cursor(self):
-        return self._span_at_cursor \
+    def get_span_at_caret(self):
+        return self._span_at_caret \
                if self._accessible else None
 
-    def get_cursor(self):
-        return self._span_at_cursor.begin() \
+    def get_caret(self):
+        return self._span_at_caret.begin() \
                if self._accessible else 0
 
     def get_text_begin_marker(self):
@@ -222,7 +218,7 @@ class AtspiTextContext(TextContext):
         """ Delete directly, without going through faking key presses. """
         self._accessible.delete_text(offset, offset + length)
 
-    def delete_text_before_cursor(self, length = 1):
+    def delete_text_before_caret(self, length = 1):
         """ Delete directly, without going through faking key presses. """
         offset = self._accessible.get_caret_offset()
         self.delete_text(offset - length, length)
@@ -233,7 +229,17 @@ class AtspiTextContext(TextContext):
         """
         self._accessible.insert_text(offset, text, -1)
 
-    def insert_text_at_cursor(self, text):
+        # Move the caret after insertion if the accessible itself
+        # hasn't done so already. This assumes the insertion begins at
+        # the current caret position, which always happens to be the case
+        # currently.
+        # Only the nautilus rename text entry appears to need this.
+        offset_before = offset
+        offset_after = self._accessible.get_caret_offset()
+        if text and offset_before == offset_after:
+            self._accessible.set_caret_offset(offset_before + len(text))
+
+    def insert_text_at_caret(self, text):
         """
         Insert directly, without going through faking key presses.
         Fails for terminal and firefox, unfortunately.
@@ -327,11 +333,11 @@ class AtspiTextContext(TextContext):
         # synchrounously notify of text insertion
         if insertion_span:
             try:
-                cursor_offset = self._accessible.get_caret_offset()
+                caret_offset = self._accessible.get_caret_offset()
             except: # gi._glib.GError
                 pass
             else:
-                self._wp.on_text_inserted(insertion_span, cursor_offset)
+                self._wp.on_text_inserted(insertion_span, caret_offset)
 
         self._update_context()
 
@@ -349,11 +355,11 @@ class AtspiTextContext(TextContext):
         if key.is_text_changing():
             keycode = 0
             if key.is_return():
-                keycode = KeyCode.KP_Enter 
+                keycode = KeyCode.KP_Enter
             else:
                 label = key.get_label()
                 if label == "C" or label == "c":
-                    keycode = KeyCode.C 
+                    keycode = KeyCode.C
 
             self._handle_key_press(keycode, mod_mask)
 
@@ -368,7 +374,7 @@ class AtspiTextContext(TextContext):
                     self._wp.commit_changes()
                 elif end_of_editing == False:
                     self._wp.discard_changes()
- 
+
     def _record_text_change(self, pos, length, insert):
         accessible = self._accessible
 
@@ -428,7 +434,7 @@ class AtspiTextContext(TextContext):
                 span.text = Atspi.Text.get_text(accessible, begin, end)
                 span.text_pos = begin
 
-           #print(self._changes)
+            #print(self._changes)
 
         self._text_changed = True
 
@@ -442,8 +448,8 @@ class AtspiTextContext(TextContext):
         if not result is None:
             (self._context,
              self._line,
-             self._line_cursor,
-             self._span_at_cursor,
+             self._line_caret,
+             self._span_at_caret,
              self._begin_of_text,
              self._begin_of_text_offset) = result
 
@@ -472,7 +478,7 @@ class InputLine(TextContext):
 
     def reset(self):
         self.line = ""
-        self.cursor = 0
+        self.caret = 0
         self.valid = True
 
         self.word_infos = {}
@@ -484,35 +490,35 @@ class InputLine(TextContext):
         return len(self.line) == 0
 
     def insert(self, s):
-        self.line   = self.line[:self.cursor] + s + self.line[self.cursor:]
-        self.move_cursor(len(s))
+        self.line   = self.line[:self.caret] + s + self.line[self.caret:]
+        self.move_caret(len(s))
 
     def delete_left(self, n=1):  # backspace
-        self.line = self.line[:self.cursor-n] + self.line[self.cursor:]
-        self.move_cursor(-n)
+        self.line = self.line[:self.caret-n] + self.line[self.caret:]
+        self.move_caret(-n)
 
     def delete_right(self, n=1): # delete
-        self.line = self.line[:self.cursor] + self.line[self.cursor+n:]
+        self.line = self.line[:self.caret] + self.line[self.caret+n:]
 
-    def move_cursor(self, n):
-        self.cursor += n
+    def move_caret(self, n):
+        self.caret += n
 
         # moving into unknown territory -> suggest reset
-        if self.cursor < 0:
-            self.cursor = 0
+        if self.caret < 0:
+            self.caret = 0
             self.valid = False
-        if self.cursor > len(self.line):
-            self.cursor = len(self.line)
+        if self.caret > len(self.line):
+            self.caret = len(self.line)
             self.valid = False
 
     def get_context(self):
-        return self.line[:self.cursor]
+        return self.line[:self.caret]
 
     def get_line(self):
         return self.line
 
-    def get_line_cursor_pos(self):
-        return self.cursor
+    def get_line_caret_pos(self):
+        return self.caret
 
     @staticmethod
     def is_printable(char):
@@ -557,9 +563,9 @@ class InputLine(TextContext):
             if   id == 'DELE':
                 self.delete_right()
             elif id == 'LEFT':
-                self.move_cursor(-1)
+                self.move_caret(-1)
             elif id == 'RGHT':
-                self.move_cursor(1)
+                self.move_caret(1)
             else:
                 end_editing = True
 
@@ -583,9 +589,9 @@ class InputLine(TextContext):
         else:
             end_editing = True
 
-        if not self.is_valid(): # cursor moved outside known range?
+        if not self.is_valid(): # caret moved outside known range?
             end_editing = True
 
-        #print end_editing,"'%s' " % self.line, self.cursor
+        #print end_editing,"'%s' " % self.line, self.caret
         return end_editing
 
