@@ -18,22 +18,21 @@
 
 from __future__ import division, print_function, unicode_literals
 
+import os
 import re
+import glob
+
+import logging
+_logger = logging.getLogger("TextDomain")
 
 try:
     from gi.repository import Atspi
 except ImportError as e:
-    _logger.info(_("Atspi unavailable, "
-                   "word suggestions not fully functional"))
+    pass
 
 from Onboard.TextChanges  import TextSpan
 from Onboard.utils        import KeyCode, Modifiers, unicode_str
 
-### Logging ###
-import logging
-_logger = logging.getLogger("TextDomain")
-###############
-
 
 class TextDomains:
     """ Collection of all recognized text domains. """
@@ -76,18 +75,34 @@ class TextDomain:
     def read_context(self, accessible):
         return NotImplementedError()
 
+    def get_text_begin_marker(self):
+        return ""
+
     def get_auto_separator(self, context):
         """
         Get word separator to add after inserting a prediction choice.
 
         Doctests:
+        # URL
         >>> d = DomainGenericText()
         >>> d.get_auto_separator("word http")
         ' '
         >>> d.get_auto_separator("word http://www")
         '.'
+
+        # filename
         >>> d.get_auto_separator("/etc")
+        '/'
+        >>> from os.path import abspath, dirname
+        >>> path = dirname(abspath("./onboard-defaults.conf.example"))
+        >>> d.get_auto_separator(path + "/onboard-defaults")
+        '.'
+        >>> d.get_auto_separator(path + "/onboard-defaults.conf")
+        '.'
+        >>> d.get_auto_separator(path + "/onboard-defaults.conf.example")
         ''
+        >>> d.get_auto_separator(path + "/onboard-defaults.conf.example.nexus7")
+        ' '
         """
         separator = " "
 
@@ -97,15 +112,67 @@ class TextDomain:
             string = strings[-1]
             if self._url_parser.is_maybe_url(string):
                 separator = self._url_parser.get_auto_separator(string)
-            else:
-                # File name?
-                if  "/" in string:
-                    separator = ""
+            elif self._url_parser._is_maybe_filename(string):
+                url = "file://" + string
+                separator = self._url_parser.get_auto_separator(url)
 
         return separator
 
-    def get_text_begin_marker(self):
-        return ""
+    def grow_learning_span(self, text_span):
+        """
+        Grow span before learning to include e.g. whole URLs.
+
+        Doctests:
+        >>> d = DomainGenericText()
+
+        # Span doesn't change for simple words
+        >>> d.grow_learning_span(TextSpan(8, 1, "word1 word2 word3"))
+        (8, 1, 'r')
+
+        # Span growing to cover a complete URL
+        >>> d.grow_learning_span(TextSpan(13, 1, "http://www.domain.org"))
+        (0, 21, 'http://www.domain.org')
+
+        # Span growing to cover multiple complete URLs
+        >>> d.grow_learning_span(TextSpan(19, 13, "http://www.domain.org word http://slashdot.org"))
+        (0, 46, 'http://www.domain.org word http://slashdot.org')
+
+        # Span growing to cover a complete filename
+        >>> d.grow_learning_span(TextSpan(10, 1, "word1 /usr/bin/bash word2"))
+        (6, 13, '/usr/bin/bash')
+
+        # Edge cases
+        >>> d.grow_learning_span(TextSpan(6, 0, "word1 /usr/bin/bash word2"))
+        (6, 0, '')
+        >>> d.grow_learning_span(TextSpan(19, 0, "word1 /usr/bin/bash word2"))
+        (19, 0, '')
+        >>> d.grow_learning_span(TextSpan(6, 1, "word1 /usr/bin/bash word2"))
+        (6, 13, '/usr/bin/bash')
+        >>> d.grow_learning_span(TextSpan(18, 1, "word1 /usr/bin/bash word2"))
+        (6, 13, '/usr/bin/bash')
+
+        # Large text with text offset > 0: returned position must be offset too.
+        >>> d.grow_learning_span(TextSpan(116, 1, "word1 /usr/bin/bash word2", 100))
+        (106, 13, '/usr/bin/bash')
+        """
+        text   = text_span.get_text()
+        offset = text_span.text_begin()
+        begin  = text_span.begin() - offset
+        end    = text_span.end() - offset
+
+        tokens, spans = self._split_growth_sections(text)
+
+        for i, s in enumerate(spans):
+            if begin < s[1] and end > s[0]: # intersects?
+
+                token = tokens[i]
+                span = spans[i]
+                if self._url_parser.is_maybe_url(token) or \
+                   self._url_parser._is_maybe_filename(token):
+                    begin = min(begin, span[0])
+                    end = max(end, span[1])
+
+        return begin+offset, end - begin, text[begin:end]
 
     def can_record_insertion(self, accessible, pos, length):
         return True
@@ -126,6 +193,33 @@ class TextDomain:
     def handle_key_press(self, keycode, mod_mask):
         return True, None  # entering_text, end_of_editing
 
+    _growth_tokens_pattern = re.compile("[^\s?#@]+", re.DOTALL)
+
+    def _split_growth_sections(self, text):
+        """
+        Split text at whitespace and other delimiters where
+        growing learning spans should stop.
+
+        Doctests:
+        >>> d = DomainGenericText()
+        >>> d._split_growth_sections("word1 www.domain.org word2. http://test")
+        (['word1', 'www.domain.org', 'word2.', 'http://test'], [(0, 5), (6, 20), (21, 27), (28, 39)])
+
+        >>> d._split_growth_sections("http://www.domain.org/?p=1#anchor")
+        (['http://www.domain.org/', 'p=1', 'anchor'], [(0, 22), (23, 26), (27, 33)])
+
+        >>> d._split_growth_sections("http://user:pass@www.domain.org")
+        (['http://user:pass', 'www.domain.org'], [(0, 16), (17, 31)])
+        """
+        matches = self._growth_tokens_pattern.finditer(text)
+        tokens = []
+        spans = []
+        for m in matches:
+            tokens.append(m.group())
+            spans.append(m.span())
+        return tokens, spans
+
+
 class DomainNOP(TextDomain):
     """ Do-nothing domain, no focused accessible. """
 
@@ -170,7 +264,7 @@ class DomainGenericText(TextDomain):
             return None
 
         line = unicode_str(r.content).replace("\n","")
-        line_cursor = max(offset - r.start_offset, 0)
+        line_caret = max(offset - r.start_offset, 0)
 
         begin = max(offset - 256, 0)
         end   = min(offset + 100, count)
@@ -178,12 +272,12 @@ class DomainGenericText(TextDomain):
 
         text = unicode_str(text)
 
-        cursor_span = TextSpan(offset, 0, text, begin)
+        caret_span = TextSpan(offset, 0, text, begin)
         context = text[:offset - begin]
         begin_of_text = begin == 0
         begin_of_text_offset = 0
 
-        return (context, line, line_cursor, cursor_span,
+        return (context, line, line_caret, caret_span,
                 begin_of_text, begin_of_text_offset)
 
     def can_spell_check(self):
@@ -238,7 +332,7 @@ class DomainTerminal(TextDomain):
                 return None
 
         # remove prompt from the current or previous lines
-        context, context_start, line, line_start, line_cursor = \
+        context, context_start, line, line_start, line_caret = \
                 self._read_after_prompt(accessible, offset)
         if context_start:
             begin_of_text = True
@@ -250,10 +344,10 @@ class DomainTerminal(TextDomain):
         # remove newlines
         context = context.replace("\n","")
 
-        #cursor_span = TextSpan(offset, 0, text, begin)
-        cursor_span = TextSpan(offset, 0, line, line_start)
+        #caret_span = TextSpan(offset, 0, text, begin)
+        caret_span = TextSpan(offset, 0, line, line_start)
 
-        result = (context, line, line_cursor, cursor_span,
+        result = (context, line, line_caret, caret_span,
                   begin_of_text, begin_of_text_offset)
         return result
 
@@ -262,12 +356,12 @@ class DomainTerminal(TextDomain):
                             Atspi.TextBoundaryType.LINE_START)
         line = unicode_str(r.content).replace("\n","")
         line_start = r.start_offset
-        line_cursor = offset - line_start
+        line_caret = offset - line_start
 
         # remove prompt from the current or previous lines
         context = ""
         context_start = None
-        l = line[:line_cursor]
+        l = line[:line_caret]
         for i in range(2):
 
             # blacklist matches? -> cancel whole context
@@ -281,7 +375,7 @@ class DomainTerminal(TextDomain):
             if i == 0:
                 line = line[context_start:] # cut prompt from input line
                 line_start  += context_start
-                line_cursor -= context_start
+                line_caret -= context_start
             if context_start:
                 break
 
@@ -291,7 +385,7 @@ class DomainTerminal(TextDomain):
                                 Atspi.TextBoundaryType.LINE_START)
             l = unicode_str(r.content).replace("\n","")
 
-        result = (context, context_start, line, line_start, line_cursor)
+        result = (context, context_start, line, line_start, line_caret)
         return result
 
     def _find_prompt(self, context):
@@ -319,7 +413,7 @@ class DomainTerminal(TextDomain):
         # Only record (for learning) when there is a known prompt in sight.
         # Problem: learning won't happen for uncommon prompts, but less random
         # junk scrolling by should enter the user model in return.
-        context, context_start, line, line_start, line_cursor = \
+        context, context_start, line, line_start, line_caret = \
                 self._read_after_prompt(accessible, offset)
         return bool(context_start)
 
@@ -376,9 +470,18 @@ class PartialURLParser:
 
     Doctests:
     >>> p = PartialURLParser()
+
     >>> p.tokenize_url('http://user:pass@www.do-mai_n.nl/path/name.ext')
     ['http', '://', 'user', ':', 'pass', '@', 'www', '.', 'do-mai_n', '.', 'nl', '/', 'path', '/', 'name', '.', 'ext']
 
+    >>> p.tokenize_url('user:pass@www.do-mai_n.nl/path/name.ext')
+    ['user', ':', 'pass', '@', 'www', '.', 'do-mai_n', '.', 'nl', '/', 'path', '/', 'name', '.', 'ext']
+
+    >>> p.tokenize_url('www.do-mai_n.nl/path/name.ext')
+    ['www', '.', 'do-mai_n', '.', 'nl', '/', 'path', '/', 'name', '.', 'ext']
+
+    >>> p.tokenize_url('www.do-mai_n.nl')
+    ['www', '.', 'do-mai_n', '.', 'nl']
     """
     _gTLDs   = ["aero", "asia", "biz", "cat", "com", "coop", "info", "int",
                "jobs", "mobi", "museum", "name", "net", "org", "pro", "tel",
@@ -437,16 +540,44 @@ class PartialURLParser:
         True
         >>> d.is_maybe_url("http://www.domain.org")
         True
+        >>> d.is_maybe_url("www.domain.org")
+        True
+        >>> d.is_maybe_url("www.domain")
+        False
+        >>> d.is_maybe_url("www")
+        False
         """
         tokens = self.tokenize_url(context)
+
+        # with scheme
         if len(tokens) >= 2:
             token  = tokens[0]
             septok = tokens[1]
             if token in self._all_schemes and septok.startswith(":"):
                 return True
+
+        # without scheme
+        if len(tokens) >= 5:
+            if tokens[1] == "." and tokens[3] == ".":
+                try:
+                    index = tokens.index("/")
+                except ValueError:
+                    index = 0
+
+                if index >= 4:
+                    hostname = tokens[:index]
+                else:
+                    hostname = tokens
+
+                if hostname[-1] in self._TLDs:
+                    return True
+
         return False
 
 
+    def _is_maybe_filename(self, string):
+            return  "/" in string
+
     def get_auto_separator(self, context):
         """
         Get word separator to add after inserting a prediction choice.
@@ -457,20 +588,53 @@ class PartialURLParser:
         '://'
         >>> p.get_auto_separator("www")
         '.'
+        >>> p.get_auto_separator("domain.org")
+        '/'
+        >>> p.get_auto_separator("www.domain.org")
+        '/'
         >>> p.get_auto_separator("http://www.domain")
         '.'
         >>> p.get_auto_separator("http://www.domain.org")
         '/'
+        >>> p.get_auto_separator("http://www.domain.co")
+        '.'
+        >>> p.get_auto_separator("http://www.domain.co.uk")
+        '/'
         >>> p.get_auto_separator("http://www.domain.org/home")
         ''
         >>> p.get_auto_separator("mailto")
         ':'
+
+        # local files
         >>> p.get_auto_separator("file")
         ':///'
         >>> p.get_auto_separator("file:///home")
+        '/'
+        >>> from os.path import abspath, dirname
+        >>> url = "file://" + dirname(abspath("./onboard-defaults.conf.example"))
+        >>> p.get_auto_separator(url + "/onboard-defaults")
+        '.'
+        >>> p.get_auto_separator(url + "/onboard-defaults.conf")
+        '.'
+        >>> p.get_auto_separator(url + "/onboard-defaults.conf.example")
         ''
+        >>> p.get_auto_separator(url + "/onboard-defaults.conf.example.nexus7")
+        ' '
+
+        # Non-existing filename: we don't know, don't guess a separator
+        >>> p.get_auto_separator("file:///tmp/onboard1234")
+        ''
+
+        # Non-existing filename: if basename has an extension assumes we're done
+        >>> p.get_auto_separator("file:///tmp/onboard1234.txt")
+        ' '
+
+        # Relative filename: we don't know current dir, return empty separator
+        >>> p.get_auto_separator("file://tmp")
+        ''
+
         """
-        separator = " " # may be entering search terms, keep space as default
+        separator = None
 
         SCHEME, PROTOCOL, DOMAIN, PATH = range(4)
         component = SCHEME
@@ -504,7 +668,9 @@ class PartialURLParser:
             if component == DOMAIN:
                 if token:
                     separator = "."
-                    if last_septok == "." and token in self._TLDs:
+                    if last_septok == "." and \
+                       token in self._TLDs and \
+                       token != "co":  # special case for co.uk TLD
                         separator = "/"
                         component = PATH
                         continue
@@ -515,5 +681,98 @@ class PartialURLParser:
             if component == PROTOCOL:
                 separator = ""
 
+        if component == PATH and not separator:
+            file_scheme = "file://"
+            if context.startswith(file_scheme):
+                filename = context[len(file_scheme):]
+                separator = self._get_filename_separator(filename)
+
+        if separator is None:
+             separator = " " # may be entering search terms, keep space as default
+
         return separator
 
+    def _get_filename_separator(self, filename):
+        """
+        Get auto separator for a partial filename.
+        """
+        separator = None
+
+        if os.path.isabs(filename):
+            files  = glob.glob(filename + "*")
+            files += glob.glob(filename + "/*") # look inside directories too
+            separator = self._get_separator_from_file_list(filename, files)
+
+        if separator is None:
+            basename = os.path.basename(filename)
+            if "." in basename:
+                separator = " "
+            else:
+                separator = ""
+
+        return separator
+
+    @staticmethod
+    def _get_separator_from_file_list(filename, files):
+        """
+        Extract separator from a list of matching filenames.
+
+        Doctests:
+        >>> p = PartialURLParser
+
+        # no matching files: return None, assume new file we can't check
+        >>> p._get_separator_from_file_list("/dir/file", [])
+
+        # complete file: we're done, continue with space separator
+        >>> p._get_separator_from_file_list("/dir/file.ext", ["/dir/file.ext"])
+        ' '
+
+        # multiple files with identical separator: return that separator
+        >>> p._get_separator_from_file_list("/dir/file",
+        ...                                ["/dir/file.ext1", "/dir/file.ext2"])
+        '.'
+
+        # multiple files with different separators: return empty separator
+        >>> p._get_separator_from_file_list("/dir/file",
+        ...                                ["/dir/file.ext", "/dir/file-ext"])
+        ''
+
+        # directory
+        >>> p._get_separator_from_file_list("/dir",
+        ...                                ["/dir/file.ext1", "/dir/file.ext2"])
+        '/'
+        >>> p._get_separator_from_file_list("/dir",
+        ...                                ["/dir", "/dir/file.ext2"])
+        '/'
+
+        # multiple extensions
+        >>> files = ["/dir/dir/file.ext1.ext2", "/dir/dir/file.ext1.ext3"]
+        >>> p._get_separator_from_file_list("/dir/dir/file", files)
+        '.'
+        >>> p._get_separator_from_file_list("/dir/dir/file.ext1", files)
+        '.'
+        >>> p._get_separator_from_file_list("/dir/dir/file.ext1.ext2", files)
+        ' '
+        """
+        separator = None
+
+        l = len(filename)
+        separators = set([f[l:l+1] for f in files \
+                         if f.startswith(filename)])
+
+        # directory?
+        if len(separators) == 2 and "/" in separators and "" in separators:
+            separator = "/"
+        # end of filename?
+        elif len(separators) == 1 and "" in separators:
+            separator = " "
+        # unambigous separator?
+        elif len(separators) == 1:
+            separator = separators.pop()
+        # multiple separators
+        elif separators:
+            separator = ""
+
+        return separator
+
+
