@@ -145,8 +145,8 @@ static PyMemberDef osk_device_event_members[] = {
     {"source_id", T_UINT, offsetof(OskDeviceEvent, source_id), READONLY, NULL },
     {"x", T_DOUBLE, offsetof(OskDeviceEvent, x), RESTRICTED, NULL },
     {"y", T_DOUBLE, offsetof(OskDeviceEvent, y), RESTRICTED, NULL },
-    {"x_root", T_DOUBLE, offsetof(OskDeviceEvent, x_root), READONLY, NULL },
-    {"y_root", T_DOUBLE, offsetof(OskDeviceEvent, y_root), READONLY, NULL },
+    {"x_root", T_DOUBLE, offsetof(OskDeviceEvent, x_root), RESTRICTED, NULL },
+    {"y_root", T_DOUBLE, offsetof(OskDeviceEvent, y_root), RESTRICTED, NULL },
     {"button", T_UINT, offsetof(OskDeviceEvent, button), READONLY, NULL },
     {"state", T_UINT, offsetof(OskDeviceEvent, state), RESTRICTED, NULL },
     {"keyval", T_UINT, offsetof(OskDeviceEvent, keyval), READONLY, NULL },
