@@ -241,6 +241,60 @@ def matmult(m, v):
 def hexstring_to_float(hexString):
     return float(int(hexString, 16))
 
+def hexcolor_to_rgba(color):
+    """
+    convert '#rrggbb' or '#rrggbbaa' to (r, g, b, a)
+
+    Doctests:
+    >>> def test(color):
+    ...     rgba = hexcolor_to_rgba(color)
+    ...     if rgba is None:
+    ...         print(repr(rgba))
+    ...     else:
+    ...         print(repr([round(c, 2) for c in rgba]))
+
+    >>> test("#1a2b3c")
+    [0.1, 0.17, 0.24, 1.0]
+
+    >>> test("#1a2b3c4d")
+    [0.1, 0.17, 0.24, 0.3]
+
+    >>> test("")
+    None
+
+    >>> test("1a2b3c")
+    None
+
+    >>> test("1a2b3c4d")
+    None
+
+    >>> test("#1a2b3c4dx")
+    None
+
+    >>> test("#1a2b3cx")
+    None
+
+    >>> test("#1a2bx")
+    None
+
+    >>> test("#1aXb3c4d")
+    None
+    """
+    rgba = None
+    n = len(color)
+    if n == 7 or n == 9:
+        try:
+            rgba = [hexstring_to_float(color[1:3])/255,
+                    hexstring_to_float(color[3:5])/255,
+                    hexstring_to_float(color[5:7])/255]
+            if n == 9:
+                rgba.append(hexstring_to_float(color[7:9])/255)
+            else:
+                rgba.append(1.0)
+        except ValueError:
+            rgba = None
+    return rgba
+
 class dictproperty(object):
     """ Property implementation for dictionaries """
 
@@ -303,15 +357,19 @@ def show_ask_string_dialog(question, parent=None):
     question_dialog.destroy()
     return text
 
-def show_confirmation_dialog(question, parent=None):
+def show_confirmation_dialog(question, parent=None, center=False):
     """
     Show this dialog to ask confirmation before executing a task.
     """
     dlg = Gtk.MessageDialog(message_type=Gtk.MessageType.QUESTION,
-                            message_format=question,
+                            text=question,
                             buttons=Gtk.ButtonsType.YES_NO)
     if parent:
         dlg.set_transient_for(parent)
+
+    if center:
+        dlg.set_position(Gtk.WindowPosition.CENTER)
+
     response = dlg.run()
     dlg.destroy()
     return response == Gtk.ResponseType.YES
@@ -684,6 +742,42 @@ class Rect:
             result.y += y_align * (rect.h - result.h)
         return result
 
+    def resize_to_aspect(self, aspect_rect):
+        """
+        Resize self to get the aspect ratio of aspect_rect.
+        """
+        if self.is_empty() or aspect_rect.is_empty():
+            return Rect()
+
+        src_aspect = aspect_rect.w / float(aspect_rect.h)
+        dst_aspect = self.w / float(self.h)
+
+        result = self.copy()
+        if dst_aspect > src_aspect:
+            result.w = self.h * src_aspect
+        else:
+            result.h = self.w / src_aspect
+        return result
+
+    def resize_to_aspect_range(self, aspect_rect, aspect_change_range):
+        """
+        Resize self to get the aspect ratio of aspect_rect, but limited
+        but the given aspect range.
+        """
+        if self.is_empty() or aspect_rect.is_empty():
+            return Rect()
+
+        r = aspect_rect
+        if r.h:
+            a0 = r.w / float(r.h)
+            a0_max = a0 * aspect_change_range[1]
+            a1 = self.w / float(self.h)
+            a = min(a1, a0_max)
+
+            r = Rect(0, 0, a, 1.0)
+            r = Rect(0, 0, a, 1.0)
+        return self.resize_to_aspect(r)
+
     def align_rect(self, rect, x_align = 0.5, y_align = 0.5):
         """
         Aligns the given rect inside of self.
